import re
from datetime import datetime
from typing import Optional, Union, List

from pydantic import BaseModel, EmailStr, field_validator, ConfigDict, Field



class TokenBase(BaseModel):
    """Base schema for tokens"""

    id: str
    name: str
    symbol: str
    decimal_places: int
    created_at: datetime
    updated_at: datetime

    model_config = ConfigDict(from_attributes=True)


class TokenCreate(BaseModel):
    """Schema to create a new token"""

    name: str
    symbol: str
    decimal_places: int = Field(18, description="Number of decimal places for the token")
    network: str
    contract_address: str



class TokenBalanceBase(BaseModel):
    """Base schema for token balances"""

    id: str
    wallet_id: str
    token_id: str
    amount: float
    created_at: datetime
    updated_at: datetime

    model_config = ConfigDict(from_attributes=True)


class TokenBalanceCreate(BaseModel):
    """Schema to add or update token balance in a wallet"""

    wallet_id: str
    token_id: str
    amount: float


class TokenData(BaseModel):
    """Schema to return token data"""

    id: str
    name: str
    symbol: str
    decimal_places: int
    created_at: datetime
    updated_at: datetime
    
    @staticmethod
    def format_date(value: datetime):
        return value.strftime("%b %d, %Y") if value else None
    
    def dict(self, *args, **kwargs):
        data = super().dict(*args, **kwargs)
        data["created_at"] = self.format_date(self.created_at)
        data["updated_at"] = self.format_date(self.updated_at)

        return data

    model_config = ConfigDict(from_attributes=True)


class TokenWithdraw(BaseModel):
    """Schema to add or update token balance in a wallet"""

    amount: float
    token: str
    address: str
    
class TokenDeposit(BaseModel):
    """Schema to add or update token balance in a wallet"""

    amount: float
    token: str
    transaction_hash: str
    
class ApproveTxn(BaseModel):
    """Schema for trx approval"""

    amount: float
    user_id: str
    token_id: str
    transaction_id: str 
    transaction_type: str
    
    
class RejectTxn(BaseModel):
    """Schema for trx rejecion"""

    amount: float
    user_id: str
    token_id: str
    transaction_id: str 
    
class SystemTokenInfo(BaseModel):
    """Schema for trx rejecion"""

    name: str
    address: str
    